<?php
/**
 * @version		1.6.1
 * @package		Joomla
 * @subpackage	Event Booking
 * @author  Tuan Pham Ngoc
 * @copyright	Copyright (C) 2010 - 2013 Ossolution Team
 * @license		GNU/GPL, see LICENSE.php
 */
// no direct access
defined('_JEXEC') or die();

/**
 * Event Booking Registrant Model
 *
 * @package		Joomla
 * @subpackage	Event Booking
 */
class EventbookingModelRegistrant extends RADModelItem
{

	/**
	 * Method to store a registrant
	 *
	 * @access	public
	 * @param	RADInput $input
	 * @return	boolean	True on success	 
	 */
	function store($input)
	{
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		$row = $this->getTable();
		$isNew = false;
		$data = $input->getData();
		if ($data['id'])
		{
			//We will need to calculate total amount here now
			$row->load($data['id']);
		}
		else
		{
			$isNew = true;
			$row->payment_method = 'os_offline';
			$row->register_date = gmdate('Y-m-d');
			$rate = EventbookingHelper::getRegistrationRate($data['event_id'], $data['number_registrants']);
			$row->total_amount = $row->amount = $rate * $data['number_registrants'];
		}
		$row->bind($data);
		$row->store();
		
		if (EB_AFFILIATE)
		{
			if ($row->published == 1)
			{
				$query->select('id, coupon_id')
					->from('#__eb_registrants')
					->where("id=$row->id AND payment_method LIKE 'os_offline%' AND check_coupon = 0");
				$db->setQuery($query);
				$coupons = $db->loadObjectList();
				if (count($coupons))
				{
					foreach ($coupons as $coupon)
					{
						$query->update('#__eb_registrants')
							->set('check_coupon=1')
							->where('id=' . $coupon->id);
						$db->setQuery($query);
						$db->query();
						$query->clear();
						
						$query->update('#__eb_coupons')
							->set('used = used + 1')
							->where('id=' . $coupon->coupon_id);
						$db->setQuery($query);
						$db->query();
						$query->clear();
					}
				}
			}
		}
		
		$registrationId = $row->id;
		if (!$isNew)
		{ //Incase editing function, we will save custom field value and registrants
			if ($row->number_registrants > 1)
			{
				$jcFields = new EventbookingHelperFields($row->event_id, false, 1);
			}
			else
			{
				$jcFields = new EventbookingHelperFields($row->event_id, false, 0);
			}
			$jcFields->saveFieldValues($row->id);
			if ($row->is_group_billing && (strpos($row->payment_method, 'os_offline') !== false))
			{
				$query->update('#__eb_registrants')
					->set('published='.(int)$row->published)
					->where('group_id='.$row->id);
				$db->setQuery($query);
				$db->query();
				$query->clear();				
			}
			if ($row->number_registrants > 1)
			{
				//Load all member custom fields
				$jcFields = new EventbookingHelperFields($row->event_id, 0, 2);
				$ids = JRequest::getVar('ids', array(), 'post');
				$ids = (array) $data['ids'];
				$fields = array(
					'event_id', 
					'first_name', 
					'last_name', 
					'organization', 
					'address', 
					'address2', 
					'city', 
					'state', 
					'country', 
					'zip', 
					'phone', 
					'fax', 
					'email', 
					'comment');
				for ($i = 0, $n = count($ids); $i < $n; $i++)
				{
					$memberId = $ids[$i];
					$row = $this->getTable();
					$row->load($memberId);
					$memberData = array();
					foreach ($fields as $field)
					{
						if ($field == 'event_id')
						{
							$memberData[$field] = (int) $data[$field];
						}
						else
						{
							$memberData[$field] = $data[$field . '_' . $memberId];
						}
					}
					$row->bind($memberData);
					$row->store();
					//Store custom field information for this member
					$jcFields->saveMemberFieldValues($memberId);
				}
			}
		}
		else
		{
			if ($row->number_registrants > 1)
			{
				for ($i = 0; $i < $row->number_registrants; $i++)
				{
					$rowMember = $this->getTable();
					$rowMember->event_id = $row->event_id;
					$rowMember->group_id = $row->id;
					$rowMember->store();
				}
			}
		}
		
		//Update total registration price when edit registrant 
		

		$row = $this->getTable();
		$row->load($registrationId);
		
		//We will need to check
		$query->clear();
		$query->select('COUNT(id)')
			->from('#__eb_registrants')
			->where('group_id='.$row->id);
			
		$db->setQuery($query);
		$total = $db->loadResult();
		$rate = EventbookingHelper::getRegistrationRate($row->event_id, $row->number_registrants);
		
		if ($total > 0)
		{ 												
			//$feeFields = new EventbookingHelperFields($row->event_id, false, 2) ;
			$feeAmount = EventbookingHelperFields::canculateGroupFee($row->id) + EventbookingHelperFields::calculateFee($row->event_id, 1);
			$totalAmount = $rate * $row->number_registrants + $feeAmount;
		}
		else
		{
			$feeAmount = EventbookingHelperFields::calculateFee($row->event_id, 0);
			$totalAmount = $rate * $row->number_registrants + $feeAmount;
		}
		
		$row->total_amount = $totalAmount;
		$row->amount = $row->total_amount - $row->discount_amount;
		$row->store();
		$input->set('id', $row->id);		
		
		return true;
	}

	/**
	 * Method to remove registrants 
	 *
	 * @access	public
	 * @return	boolean	True on success	 
	 */
	function delete($cid = array())
	{
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		$row = $this->getTable();
		if (count($cid))
		{
			foreach ($cid as $registrantId)
			{
				$row->load($registrantId);
				if ($row->group_id > 0)
				{
					$query->update('#__eb_registrants')
						->set('number_registrants = number_registrants -1')
						->where('id='.$row->group_id);
					$db->setQuery($query);
					$db->query();					
					$query->clear();
					
					$query->select('number_registrants')
						->from('#__eb_registrants')
						->where('id='.$row->group_id);					
					$db->setQuery($query);
					$numberRegistrants = (int) $db->loadResult();
					$query->clear();
					if ($numberRegistrants == 0)
					{						
						$query->delete('#__eb_field_values')
							->where('registrant_id='.$row->group_id);																			
						$db->setQuery($query);
						$db->query();						
						$query->clear();
						
						$sql = 'DELETE FROM #__eb_registrants WHERE id = ' . $row->group_id;
						$db->setQuery($sql);
						$db->query();
						$query->clear();
					}
				}
			}
			$cids = implode(',', $cid);
			$query->select('id')
				->from('#__eb_registrants')
				->where('group_id IN (' . $cids . ')');						
			$db->setQuery($query);						
			$cid = array_merge($cid, $db->loadColumn());
			$query->clear();
			
			$registrantIds = implode(',', $cid);
			
			$query->delete('#__eb_field_values')
				->where('registrant_id IN (' . $registrantIds . ')');
			$db->setQuery($query);
			$db->query();			
			$query->clear();

			$query->delete('#__eb_registrants')
			->where('id IN (' . $registrantIds . ')');						
			$db->setQuery($query);
			$db->query();
		}
		return true;
	}

	/**
	 * Publish / unpublish a registrant 
	 *
	 * @param array $cid
	 * @param int $state
	 */
	function publish($cid, $state)
	{
		if (($state == 1) && count($cid))
		{
			$config = EventbookingHelper::getConfig();
			$row = JTable::getInstance('EventBooking', 'Registrant');
			foreach ($cid as $registrantId)
			{
				$row->load($registrantId);
				if (!$row->published)
				{
					EventbookingHelper::sendRegistrationApprovedEmail($row, $config);
				}
			}
		}
        $db = $this->getDbo();
		$cids = implode(',', $cid);
		$sql = " UPDATE #__eb_registrants SET published=$state WHERE id IN ($cids) OR group_id IN ($cids) AND payment_method LIKE 'os_offline%' ";
		$db->setQuery($sql);
		if (!$db->query())
			return false;
		if (EB_AFFILIATE)
		{
			if ($state == 1)
			{
				$sql = "SELECT id, coupon_id FROM #__eb_registrants WHERE id IN ($cids) AND payment_method LIKE 'os_offline%' AND check_coupon = 0";
				$db->setQuery($sql);
				$coupons = $db->loadObjectList();
				if (count($coupons))
				{
					foreach ($coupons as $coupon)
					{
						
						$sql = " UPDATE #__eb_registrants SET check_coupon = 1 WHERE id = $coupon->id";
						$db->setQuery($sql);
						$db->query();
						
						$sql = 'UPDATE #__eb_coupons SET used = used + 1 WHERE id=' . (int) $coupon->coupon_id;
						$db->setQuery($sql);
						$db->query();
					}
				}
			}
		}
		return true;
	}
}